#ifndef RULESET_STATE_H
#define RULESET_STATE_H

typedef struct creature {
    short               pos;            /* creature's location */
    unsigned char       id;             /* type of creature */
    unsigned char       dir;            /* current direction of creature */
    signed char         moving;         /* positional offset of creature */
    signed char         frame;          /* explicit animation index */
    unsigned char       hidden;         /* TRUE if creature is invisible */
    unsigned char       state;          /* internal state value */
    unsigned char       tdir;           /* internal state value */
} creature;

/* Status information specific to the MS game logic.
 */
struct msstate {
    unsigned char       chipwait;       /* ticks since Chip's last movement */
    unsigned char       chipstatus;     /* Chip's status (one of CHIP_*) */
    unsigned char       controllerdir;  /* current controller direction */
    unsigned char       lastslipdir;    /* Chip's last involuntary movement */
    unsigned char       completed;      /* level completed successfully */
    short               goalpos;        /* mouse spot to move Chip towards */
    signed char         xviewoffset;    /* offset of map view center */
    signed char         yviewoffset;    /*   position from position of Chip */
};

/* Status information specific to the Lynx game logic.
 */
struct lxstate {
    creature           *chiptocreature; /* is Chip colliding with a creature */
#ifndef NSPIRE_BUILD
    creature           *crend;          /* near the end of the creature list */
#endif
    short               chiptopos;      /*   just starting to move itself? */
    unsigned char       prng1;          /* the values used to make the */
    unsigned char       prng2;          /*   pseudorandom number sequence */
    signed char         xviewoffset;    /* offset of map view center */
    signed char         yviewoffset;    /*   position from position of Chip */
    unsigned char       endgametimer;   /* end-game countdown timer */
    unsigned char       togglestate;    /* extra state of the toggle walls */
    unsigned char       completed;      /* level completed successfully */
    unsigned char       stuck;          /* Chip is stuck on a teleport */
    unsigned char       pushing;        /* Chip is pushing against something */
    unsigned char       couldntmove;    /* can't-move sound has been played */
    unsigned char       mapbreached;    /* Border of map has been breached */
};

#endif

