/* err.c: Error handling and reporting.
 *
 * Copyright (C) 2001-2006 by Brian Raiter, under the GNU General Public
 * License. No warranty. See COPYING for details.
 */

#ifndef NSPIRE_BUILD
#include	<stdlib.h>
#include	<stdarg.h>
#else
#include "os.h"
#endif
#include	"oshw.h"
#include	"err.h"

/* "Hidden" arguments to _warn, _errmsg, and _die.
 */
char const      *_err_cfile = NULL;
unsigned long	_err_lineno = 0;

/* Log a warning message.
 */
void _warn(char const *fmt, ...)
{
    va_list	args;

    va_start(args, fmt);
    usermessage(NOTIFY_LOG, NULL, _err_cfile, _err_lineno, fmt, args);
    va_end(args);
    _err_cfile = NULL;
    _err_lineno = 0;
}

/* Display an error message to the user.
 */
void _errmsg(char const *prefix, char const *fmt, ...)
{
    va_list	args;

    va_start(args, fmt);
    usermessage(NOTIFY_ERR, prefix, _err_cfile, _err_lineno, fmt, args);
    va_end(args);
    _err_cfile = NULL;
    _err_lineno = 0;
}

/* Display an error message to the user and exit.
 */
void _die(char const *fmt, ...)
{
    va_list	args;

    va_start(args, fmt);
    usermessage(NOTIFY_DIE, NULL, _err_cfile, _err_lineno, fmt, args);
    va_end(args);
#ifndef NSPIRE_BUILD
    exit(EXIT_FAILURE);
#endif
}

int ReadFromStr(char* src, char* format, ... )
{
  printf("ReadFromStr called\n");
  va_list ap;
  float *f;
  int conv = 0, *i, index;
  char *a, *fp, *sp = src, buf[BUFSIZ] = {'\0'};
 
  va_start ( ap, format );
  for ( fp = format; *fp != '\0'; fp++ ) {
    for ( index = 0; *sp != '\0' && *sp != ' '; index++ )
      buf[index] = *sp++;
    while ( *sp == ' ' ) sp++;
    while ( *fp != '%' ) fp++;
    if ( *fp == '%' ) {
      switch ( *++fp ) {
      case 'i':
        i = va_arg ( ap, int * );
        *i = atoi ( buf );
        break;
      case 'f':
        f = va_arg ( ap, float * );
        *f = (float)atof ( buf );
        break;
      case 's':
        a = va_arg ( ap, char * );
        strncpy ( a, buf, strlen ( buf ) + 1 );
        break;
      }
      conv++;
    }
  }
  va_end ( ap );
  return conv;
}

